@@ -1,10 +1,12 @@
-/* SPDX-License-Identifier: BSD-2 */
+/* SPDX-License-Identifier: BSD-2-Clause */
 /*******************************************************************************
  * Copyright 2017-2018, Fraunhofer SIT sponsored by Infineon Technologies AG
  * All rights reserved.
  ******************************************************************************/
 
-#define _GNU_SOURCE
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 #include <openssl/evp.h>
 #include <openssl/aes.h>
@@ -19,21 +21,27 @@
 
 #include "esys_iutil.h"
 #include "esys_mu.h"
-#define LOGMODULE esys
+#define LOGMODULE esys_crypto
 #include "util/log.h"
-#include "esys_crypto_ossl.h"
+#include "util/aux_util.h"
 
-static ENGINE *engine = NULL;
+#if OPENSSL_VERSION_NUMBER >= 0x10101000L
+#define EC_POINT_set_affine_coordinates_tss(group, tpm_pub_key, bn_x, bn_y, dmy) \
+        EC_POINT_set_affine_coordinates(group, tpm_pub_key, bn_x, bn_y, dmy)
 
-ENGINE *get_engine()
-{
-    if (engine)
-        return engine;
-    engine = ENGINE_by_id("openssl");
-    return engine;
-}
+#define EC_POINT_get_affine_coordinates_tss(group, tpm_pub_key, bn_x, bn_y, dmy) \
+        EC_POINT_get_affine_coordinates(group, tpm_pub_key, bn_x, bn_y, dmy)
 
-int BN_bn2binpad(const BIGNUM *bn, unsigned char *bin, int bin_length)
+#else
+#define EC_POINT_set_affine_coordinates_tss(group, tpm_pub_key, bn_x, bn_y, dmy) \
+        EC_POINT_set_affine_coordinates_GFp(group, tpm_pub_key, bn_x, bn_y, dmy)
+
+#define EC_POINT_get_affine_coordinates_tss(group, tpm_pub_key, bn_x, bn_y, dmy) \
+        EC_POINT_get_affine_coordinates_GFp(group, tpm_pub_key, bn_x, bn_y, dmy)
+#endif /* OPENSSL_VERSION_NUMBER >= 0x10101000L */
+
+static int
+iesys_bn2binpad(const BIGNUM *bn, unsigned char *bin, int bin_length)
 {
     int len_bn = BN_num_bytes(bn);
     int offset = bin_length - len_bn;
@@ -62,30 +70,6 @@ typedef struct _IESYS_CRYPTO_CONTEXT {
     };
 } IESYS_CRYPTOSSL_CONTEXT;
 
-size_t
-hash_get_digest_size(TPM2_ALG_ID hashAlg)
-{
-    switch (hashAlg) {
-    case TPM2_ALG_SHA1:
-        return TPM2_SHA1_DIGEST_SIZE;
-        break;
-    case TPM2_ALG_SHA256:
-        return TPM2_SHA256_DIGEST_SIZE;
-        break;
-    case TPM2_ALG_SHA384:
-        return TPM2_SHA384_DIGEST_SIZE;
-        break;
-    case TPM2_ALG_SHA512:
-        return TPM2_SHA512_DIGEST_SIZE;
-        break;
-    case TPM2_ALG_SM3_256:
-        return TPM2_SM3_256_DIGEST_SIZE;
-        break;
-    default:
-        return 0;
-    }
-}
-
 const EVP_MD *
 get_ossl_hash_md(TPM2_ALG_ID hashAlg)
 {
@@ -135,7 +119,7 @@ iesys_cryptossl_hash_start(IESYS_CRYPTO_CONTEXT_BLOB ** context,
                    "Unsupported hash algorithm (%"PRIu16")", cleanup, hashAlg);
     }
 
-    if (!(mycontext->hash.hash_len = hash_get_digest_size(hashAlg))) {
+    if (iesys_crypto_hash_get_digest_size(hashAlg, &mycontext->hash.hash_len)) {
         goto_error(r, TSS2_ESYS_RC_NOT_IMPLEMENTED,
                    "Unsupported hash algorithm (%"PRIu16")", cleanup, hashAlg);
     }
@@ -144,10 +128,9 @@ iesys_cryptossl_hash_start(IESYS_CRYPTO_CONTEXT_BLOB ** context,
         goto_error(r, TSS2_ESYS_RC_GENERAL_FAILURE, "Error EVP_MD_CTX_create", cleanup);
     }
 
-    if (1 != EVP_DigestInit_ex(mycontext->hash.ossl_context,
-                               mycontext->hash.ossl_hash_alg,
-                               get_engine())) {
-        goto_error(r, TSS2_ESYS_RC_GENERAL_FAILURE, "Errror EVP_DigestInit_ex", cleanup);
+    if (1 != EVP_DigestInit(mycontext->hash.ossl_context,
+                               mycontext->hash.ossl_hash_alg)) {
+        goto_error(r, TSS2_ESYS_RC_GENERAL_FAILURE, "Errror EVP_DigestInit", cleanup);
     }
 
     *context = (IESYS_CRYPTO_CONTEXT_BLOB *) mycontext;
@@ -249,13 +232,13 @@ iesys_cryptossl_hash_finish(IESYS_CRYPTO_CONTEXT_BLOB ** context,
         return_error(TSS2_ESYS_RC_BAD_SIZE, "Buffer too small");
     }
 
-    if (1 != EVP_DigestFinal_ex(mycontext->hash.ossl_context, buffer, &digest_size)) {
+    if (1 != EVP_DigestFinal(mycontext->hash.ossl_context, buffer, &digest_size)) {
         return_error(TSS2_ESYS_RC_GENERAL_FAILURE, "Ossl error.");
     }
 
     if (digest_size != mycontext->hash.hash_len) {
         return_error(TSS2_ESYS_RC_GENERAL_FAILURE,
-                     "Invalid size computed by EVP_DigestFinal_ex");
+                     "Invalid size computed by EVP_DigestFinal");
     }
 
     LOGBLOB_TRACE(buffer, mycontext->hash.hash_len, "read hash result");
@@ -300,7 +283,7 @@ iesys_cryptossl_hash_abort(IESYS_CRYPTO_CONTEXT_BLOB ** context)
  * The context will be created and initialized according to the hash function
  * and the used HMAC key.
  * @param[out] context The created context (callee-allocated).
- * @param[in] hmacAlg The hash algorithm for the HMAC computation.
+ * @param[in] hashAlg The hash algorithm for the HMAC computation.
  * @param[in] key The byte buffer of the HMAC key.
  * @param[in] size The size of the HMAC key.
  * @retval TSS2_RC_SUCCESS on success.
@@ -316,7 +299,6 @@ iesys_cryptossl_hmac_start(IESYS_CRYPTO_CONTEXT_BLOB ** context,
     TSS2_RC r = TSS2_RC_SUCCESS;
     EVP_PKEY *hkey = NULL;
 
-
     LOG_TRACE("called for context-pointer %p and hmacAlg %d", context, hashAlg);
     LOGBLOB_TRACE(key, size, "Starting  hmac with");
     if (context == NULL || key == NULL) {
@@ -331,7 +313,7 @@ iesys_cryptossl_hmac_start(IESYS_CRYPTO_CONTEXT_BLOB ** context,
                    "Unsupported hash algorithm (%"PRIu16")", cleanup, hashAlg);
     }
 
-    if (!(mycontext->hmac.hmac_len = hash_get_digest_size(hashAlg))) {
+    if (iesys_crypto_hash_get_digest_size(hashAlg, &mycontext->hmac.hmac_len)) {
         goto_error(r, TSS2_ESYS_RC_GENERAL_FAILURE,
                    "Unsupported hash algorithm (%"PRIu16")", cleanup, hashAlg);
     }
@@ -341,13 +323,13 @@ iesys_cryptossl_hmac_start(IESYS_CRYPTO_CONTEXT_BLOB ** context,
                    "Error EVP_MD_CTX_create", cleanup);
     }
 
-    if (!(hkey = EVP_PKEY_new_mac_key(EVP_PKEY_HMAC, get_engine(), key, size))) {
+    if (!(hkey = EVP_PKEY_new_mac_key(EVP_PKEY_HMAC, NULL, key, size))) {
         goto_error(r, TSS2_ESYS_RC_GENERAL_FAILURE,
                    "EVP_PKEY_new_mac_key", cleanup);
     }
 
     if(1 != EVP_DigestSignInit(mycontext->hmac.ossl_context, NULL,
-                               mycontext->hmac.ossl_hash_alg, get_engine(), hkey)) {
+                               mycontext->hmac.ossl_hash_alg, NULL, hkey)) {
         goto_error(r, TSS2_ESYS_RC_GENERAL_FAILURE,
                    "DigestSignInit", cleanup);
     }
@@ -356,6 +338,8 @@ iesys_cryptossl_hmac_start(IESYS_CRYPTO_CONTEXT_BLOB ** context,
 
     *context = (IESYS_CRYPTO_CONTEXT_BLOB *) mycontext;
 
+    EVP_PKEY_free(hkey);
+
     return TSS2_RC_SUCCESS;
 
  cleanup:
@@ -448,7 +432,7 @@ iesys_cryptossl_hmac_finish(IESYS_CRYPTO_CONTEXT_BLOB ** context,
     IESYS_CRYPTOSSL_CONTEXT *mycontext =
         (IESYS_CRYPTOSSL_CONTEXT *) * context;
     if (mycontext->type != IESYS_CRYPTOSSL_TYPE_HMAC) {
-        return_error(TSS2_ESYS_RC_BAD_VALUE, "bad context");
+        return_error(TSS2_ESYS_RC_BAD_REFERENCE, "bad context");
     }
 
     if (*size < mycontext->hmac.hmac_len) {
@@ -526,20 +510,31 @@ iesys_cryptossl_hmac_abort(IESYS_CRYPTO_CONTEXT_BLOB ** context)
  * @param[out] nonce The TPM2B structure for the random data (caller-allocated).
  * @param[in] num_bytes The number of bytes to be generated.
  * @retval TSS2_RC_SUCCESS on success.
+ *
+ * NOTE: the TPM should not be used to obtain the random data
  */
 TSS2_RC
 iesys_cryptossl_random2b(TPM2B_NONCE * nonce, size_t num_bytes)
 {
+    const RAND_METHOD *rand_save = RAND_get_rand_method();
+
     if (num_bytes == 0) {
         nonce->size = sizeof(TPMU_HA);
     } else {
         nonce->size = num_bytes;
     }
+
+#if OPENSSL_VERSION_NUMBER >= 0x10100000L
+    RAND_set_rand_method(RAND_OpenSSL());
+#else
+    RAND_set_rand_method(RAND_SSLeay());
+#endif
     if (1 != RAND_bytes(&nonce->buffer[0], nonce->size)) {
+        RAND_set_rand_method(rand_save);
         return_error(TSS2_ESYS_RC_GENERAL_FAILURE,
                      "Failure in random number generator.");
     }
-
+    RAND_set_rand_method(rand_save);
     return TSS2_RC_SUCCESS;
 }
 
@@ -547,7 +542,7 @@ iesys_cryptossl_random2b(TPM2B_NONCE * nonce, size_t num_bytes)
  *
  * Encrypting a buffer using a public key is used for example during
  * Esys_StartAuthSession in order to encrypt the salt value.
- * @param[in] key The key to be used for encryption.
+ * @param[in] pub_tpm_key The key to be used for encryption.
  * @param[in] in_size The size of the buffer to be encrypted.
  * @param[in] in_buffer The data buffer to be encrypted.
  * @param[in] max_out_size The maximum size for the output encrypted buffer.
@@ -566,22 +561,34 @@ iesys_cryptossl_pk_encrypt(TPM2B_PUBLIC * pub_tpm_key,
                            BYTE * out_buffer,
                            size_t * out_size, const char *label)
 {
+    const RAND_METHOD *rand_save = RAND_get_rand_method();
+#if OPENSSL_VERSION_NUMBER >= 0x10100000L
+    RAND_set_rand_method(RAND_OpenSSL());
+#else
+    RAND_set_rand_method(RAND_SSLeay());
+#endif
+
     TSS2_RC r = TSS2_RC_SUCCESS;
     const EVP_MD * hashAlg = NULL;
     RSA * rsa_key = NULL;
     EVP_PKEY *evp_rsa_key = NULL;
     EVP_PKEY_CTX *ctx = NULL;
-    BIGNUM* bne = BN_new();
+    BIGNUM* bne = NULL;
     int padding;
-#if OPENSSL_VERSION_NUMBER >= 0x10100000L
-    BIGNUM *n = NULL;
-#endif
+    char *label_copy = NULL;
+
     if (!(hashAlg = get_ossl_hash_md(pub_tpm_key->publicArea.nameAlg))) {
         LOG_ERROR("Unsupported hash algorithm (%"PRIu16")",
                   pub_tpm_key->publicArea.nameAlg);
+        RAND_set_rand_method(rand_save);
         return TSS2_ESYS_RC_NOT_IMPLEMENTED;
     }
 
+    if (!(bne = BN_new())) {
+        goto_error(r, TSS2_ESYS_RC_MEMORY,
+                   "Could not allocate Big Number", cleanup);
+    }
+
     switch (pub_tpm_key->publicArea.parameters.rsaDetail.scheme.scheme) {
     case TPM2_ALG_NULL:
         padding = RSA_NO_PADDING;
@@ -623,10 +630,14 @@ iesys_cryptossl_pk_encrypt(TPM2B_PUBLIC * pub_tpm_key,
                    "Could not create evp key.", cleanup);
     }
 #if OPENSSL_VERSION_NUMBER < 0x10100000L
-    rsa_key->n = BN_bin2bn(pub_tpm_key->publicArea.unique.rsa.buffer,
+    if (!BN_bin2bn(pub_tpm_key->publicArea.unique.rsa.buffer,
                            pub_tpm_key->publicArea.unique.rsa.size,
-                           NULL);
+                           rsa_key->n)) {
+        goto_error(r, TSS2_ESYS_RC_GENERAL_FAILURE,
+                   "Could not create rsa n.", cleanup);
+    }
 #else
+    BIGNUM *n = NULL;
     if (!(n = BN_bin2bn(pub_tpm_key->publicArea.unique.rsa.buffer,
                         pub_tpm_key->publicArea.unique.rsa.size,
                         NULL))) {
@@ -645,7 +656,7 @@ iesys_cryptossl_pk_encrypt(TPM2B_PUBLIC * pub_tpm_key,
                    "Could not set rsa key.", cleanup);
     }
 
-    if (!(ctx = EVP_PKEY_CTX_new(evp_rsa_key, get_engine()))) {
+    if (!(ctx = EVP_PKEY_CTX_new(evp_rsa_key, NULL))) {
         goto_error(r, TSS2_ESYS_RC_GENERAL_FAILURE,
                    "Could not create evp context.", cleanup);
     }
@@ -660,7 +671,14 @@ iesys_cryptossl_pk_encrypt(TPM2B_PUBLIC * pub_tpm_key,
                    "Could not set RSA passing.", cleanup);
     }
 
-    if (1 != EVP_PKEY_CTX_set0_rsa_oaep_label(ctx, label, strlen(label)+1)) {
+    label_copy = OPENSSL_strdup(label);
+    if (!label_copy) {
+        goto_error(r, TSS2_ESYS_RC_MEMORY,
+                   "Could not duplicate OAEP label", cleanup);
+    }
+
+    if (1 != EVP_PKEY_CTX_set0_rsa_oaep_label(ctx, label_copy, strlen(label_copy)+1)) {
+        OPENSSL_free(label_copy);
         goto_error(r, TSS2_ESYS_RC_GENERAL_FAILURE,
                    "Could not set RSA label.", cleanup);
     }
@@ -687,16 +705,14 @@ iesys_cryptossl_pk_encrypt(TPM2B_PUBLIC * pub_tpm_key,
                    "Could not encrypt data.", cleanup);
     }
 
-    return TSS2_RC_SUCCESS;
+    r = TSS2_RC_SUCCESS;
 
  cleanup:
     OSSL_FREE(ctx, EVP_PKEY_CTX);
-    OSSL_FREE(rsa_key, RSA);
     OSSL_FREE(evp_rsa_key, EVP_PKEY);
+    OSSL_FREE(rsa_key, RSA);
     OSSL_FREE(bne, BN);
-#if OPENSSL_VERSION_NUMBER >= 0x10100000L
-    OSSL_FREE(n, BN);
-#endif
+    RAND_set_rand_method(rand_save);
     return r;
 }
 
@@ -708,16 +724,13 @@ iesys_cryptossl_pk_encrypt(TPM2B_PUBLIC * pub_tpm_key,
  * @retval TSS2_RC_SUCCESS on success.
  * @retval TSS2_ESYS_RC_GENERAL_FAILURE The internal crypto engine failed.
  */
-TSS2_RC
+static TSS2_RC
 tpm_pub_to_ossl_pub(EC_GROUP *group, TPM2B_PUBLIC *key, EC_POINT **tpm_pub_key)
 {
 
     TSS2_RC r = TSS2_RC_SUCCESS;
     BIGNUM *bn_x = NULL;
     BIGNUM *bn_y = NULL;
-    BN_CTX *bctx = NULL;
-
-    bctx = BN_CTX_new();
 
     /* Create the big numbers for the coordinates of the point */
     if (!(bn_x = BN_bin2bn(&key->publicArea.unique.ecc.x.buffer[0],
@@ -740,15 +753,15 @@ tpm_pub_to_ossl_pub(EC_GROUP *group, TPM2B_PUBLIC *key, EC_POINT **tpm_pub_key)
                    "Create point.", cleanup);
     }
 
-    if (1 != EC_POINT_set_affine_coordinates_GFp(group,
+    if (1 != EC_POINT_set_affine_coordinates_tss(group,
                                                  *tpm_pub_key, bn_x,
-                                                 bn_y, bctx)) {
+                                                 bn_y, NULL)) {
         OSSL_FREE(*tpm_pub_key, EC_POINT);
         goto_error(r, TSS2_ESYS_RC_GENERAL_FAILURE,
                    "Set affine coordinates", cleanup);
     }
 
-    if (1 != EC_POINT_is_on_curve(group, *tpm_pub_key, bctx)) {
+    if (1 != EC_POINT_is_on_curve(group, *tpm_pub_key, NULL)) {
         goto_error(r, TSS2_ESYS_RC_GENERAL_FAILURE,
                    "The TPM point is not on the curve", cleanup);
     }
@@ -756,7 +769,6 @@ tpm_pub_to_ossl_pub(EC_GROUP *group, TPM2B_PUBLIC *key, EC_POINT **tpm_pub_key)
  cleanup:
     OSSL_FREE(bn_x, BN);
     OSSL_FREE(bn_y, BN);
-    OSSL_FREE(bctx, BN_CTX);
 
     return r;
 }
@@ -787,11 +799,11 @@ iesys_cryptossl_get_ecdh_point(TPM2B_PUBLIC *key,
                                size_t * out_size)
 {
     TSS2_RC r = TSS2_RC_SUCCESS;
-    BN_CTX *bctx = NULL;                  /* Context used for big number operations */
     EC_GROUP *group = NULL;               /* Group defines the used curve */
     EC_KEY *eph_ec_key = NULL;            /* Ephemeral ec key of application */
     const EC_POINT *eph_pub_key = NULL;   /* Public part of ephemeral key */
     EC_POINT *tpm_pub_key = NULL;         /* Public part of TPM key */
+    EC_POINT *mul_eph_tpm = NULL;
     BIGNUM *bn_x = NULL;
     BIGNUM *bn_y = NULL;
     size_t key_size;
@@ -806,7 +818,7 @@ iesys_cryptossl_get_ecdh_point(TPM2B_PUBLIC *key,
         break;
     case TPM2_ECC_NIST_P224:
         curveId = NID_secp224r1;
-        key_size = 38;
+        key_size = 28;
         break;
     case TPM2_ECC_NIST_P256:
         curveId = NID_X9_62_prime256v1;
@@ -848,17 +860,12 @@ iesys_cryptossl_get_ecdh_point(TPM2B_PUBLIC *key,
         goto_error(r, TSS2_ESYS_RC_GENERAL_FAILURE, "Get public key", cleanup);
     }
 
-    if (1 != EC_POINT_is_on_curve(group, eph_pub_key, bctx)) {
+    if (1 != EC_POINT_is_on_curve(group, eph_pub_key, NULL)) {
         goto_error(r, TSS2_ESYS_RC_GENERAL_FAILURE,
                    "Ephemeral public key is on curve",cleanup);
     }
 
     /* Write affine coordinates of ephemeral pub key to TPM point Q */
-    if (!(bctx = BN_CTX_new())) {
-        goto_error(r, TSS2_ESYS_RC_GENERAL_FAILURE,
-                   "Create bignum context", cleanup);
-    }
-
     if (!(bn_x = BN_new())) {
         goto_error(r, TSS2_ESYS_RC_GENERAL_FAILURE, "Create bignum", cleanup);
     }
@@ -867,18 +874,18 @@ iesys_cryptossl_get_ecdh_point(TPM2B_PUBLIC *key,
         goto_error(r, TSS2_ESYS_RC_GENERAL_FAILURE, "Create bignum", cleanup);
     }
 
-    if (1 != EC_POINT_get_affine_coordinates_GFp(group, eph_pub_key, bn_x,
-                                                 bn_y, bctx)) {
+    if (1 != EC_POINT_get_affine_coordinates_tss(group, eph_pub_key, bn_x,
+                                                 bn_y, NULL)) {
         goto_error(r, TSS2_ESYS_RC_GENERAL_FAILURE,
                    "Get affine x coordinate", cleanup);
     }
 
-    if (1 != BN_bn2binpad(bn_x, &Q->x.buffer[0], key_size)) {
+    if (1 != iesys_bn2binpad(bn_x, &Q->x.buffer[0], key_size)) {
         goto_error(r, TSS2_ESYS_RC_GENERAL_FAILURE,
                    "Write big num byte buffer", cleanup);
     }
 
-    if (1 != BN_bn2binpad(bn_y, &Q->y.buffer[0], key_size)) {
+    if (1 != iesys_bn2binpad(bn_y, &Q->y.buffer[0], key_size)) {
         goto_error(r, TSS2_ESYS_RC_GENERAL_FAILURE,
                    "Write big num byte buffer", cleanup);
     }
@@ -891,7 +898,6 @@ iesys_cryptossl_get_ecdh_point(TPM2B_PUBLIC *key,
     goto_if_error(r, "Convert TPM pub point to ossl pub point", cleanup);
 
     /* Multiply the ephemeral private key with TPM public key */
-    EC_POINT *mul_eph_tpm = NULL;
     const BIGNUM * eph_priv_key = EC_KEY_get0_private_key(eph_ec_key);
 
     if (!(mul_eph_tpm = EC_POINT_new(group))) {
@@ -899,19 +905,19 @@ iesys_cryptossl_get_ecdh_point(TPM2B_PUBLIC *key,
     }
 
     if (1 != EC_POINT_mul(group, mul_eph_tpm, NULL,
-                          tpm_pub_key, eph_priv_key, bctx)) {
+                          tpm_pub_key, eph_priv_key, NULL)) {
         goto_error(r, TSS2_ESYS_RC_GENERAL_FAILURE,
                    "ec point multiplication", cleanup);
     }
 
     /* Write the x-part of the affine coordinate to Z */
-    if (1 != EC_POINT_get_affine_coordinates_GFp(group, mul_eph_tpm, bn_x,
-                                                 bn_y, bctx)) {
+    if (1 != EC_POINT_get_affine_coordinates_tss(group, mul_eph_tpm, bn_x,
+                                                 bn_y, NULL)) {
         goto_error(r, TSS2_ESYS_RC_GENERAL_FAILURE,
                    "Get affine x coordinate", cleanup);
     }
 
-    if (1 != BN_bn2binpad(bn_x, &Z->buffer[0], key_size)) {
+    if (1 != iesys_bn2binpad(bn_x, &Z->buffer[0], key_size)) {
         goto_error(r, TSS2_ESYS_RC_GENERAL_FAILURE,
                    "Write big num byte buffer", cleanup);
     }
@@ -925,6 +931,8 @@ iesys_cryptossl_get_ecdh_point(TPM2B_PUBLIC *key,
     *out_size = offset;
 
  cleanup:
+    OSSL_FREE(mul_eph_tpm, EC_POINT);
+    OSSL_FREE(tpm_pub_key, EC_POINT);
     OSSL_FREE(group,EC_GROUP);
     OSSL_FREE(eph_ec_key, EC_KEY);
     /* Note: free of eph_pub_key already done by free of eph_ec_key */
@@ -940,11 +948,10 @@ iesys_cryptossl_get_ecdh_point(TPM2B_PUBLIC *key,
  * @param[in] key_bits Key size in bits.
  * @param[in] tpm_mode Block cipher mode of opertion in TSS2 notation (CFB).
  *            For parameter encryption only CFB can be used.
- * @param[in] blk_len Length Block length of AES.
  * @param[in,out] buffer Data to be encrypted. The encrypted date will be stored
  *                in this buffer.
  * @param[in] buffer_size size of data to be encrypted.
- * @param[in] iv The initialization vector. The size is equal to blk_len.
+ * @param[in] iv The initialization vector.
  * @retval TSS2_RC_SUCCESS on success, or TSS2_ESYS_RC_BAD_VALUE and
  * @retval TSS2_ESYS_RC_BAD_REFERENCE for invalid parameters,
  * @retval TSS2_ESYS_RC_GENERAL_FAILURE for errors of the crypto library.
@@ -954,7 +961,6 @@ iesys_cryptossl_sym_aes_encrypt(uint8_t * key,
                                 TPM2_ALG_ID tpm_sym_alg,
                                 TPMI_AES_KEY_BITS key_bits,
                                 TPM2_ALG_ID tpm_mode,
-                                size_t blk_len,
                                 uint8_t * buffer,
                                 size_t buffer_size,
                                 uint8_t * iv)
@@ -970,9 +976,6 @@ iesys_cryptossl_sym_aes_encrypt(uint8_t * key,
 
     LOGBLOB_TRACE(buffer, buffer_size, "IESYS AES input");
 
-    /* Parameter blk_len needed for other crypto libraries */
-    (void)blk_len;
-
     if (key_bits == 128 && tpm_mode == TPM2_ALG_CFB)
         cipher_alg = EVP_aes_128_cfb();
     else if (key_bits == 192 && tpm_mode == TPM2_ALG_CFB)
@@ -996,20 +999,17 @@ iesys_cryptossl_sym_aes_encrypt(uint8_t * key,
                    "Initialize cipher context", cleanup);
     }
 
-    if (1 != EVP_EncryptInit_ex(ctx, cipher_alg, get_engine(), key, iv)) {
+    if (1 != EVP_EncryptInit(ctx, cipher_alg,key, iv)) {
         goto_error(r, TSS2_ESYS_RC_GENERAL_FAILURE,
                    "Initialize cipher operation", cleanup);
     }
-    if (1 != EVP_EncryptInit_ex(ctx, NULL, get_engine(), key, iv)) {
-        goto_error(r, TSS2_ESYS_RC_GENERAL_FAILURE, "Set key and iv", cleanup);
-    }
 
     /* Perform the encryption */
     if (1 != EVP_EncryptUpdate(ctx, buffer, &cipher_len, buffer, buffer_size)) {
         goto_error(r, TSS2_ESYS_RC_GENERAL_FAILURE, "Encrypt update", cleanup);
     }
 
-    if (1 != EVP_EncryptFinal_ex(ctx, buffer, &cipher_len)) {
+    if (1 != EVP_EncryptFinal(ctx, buffer, &cipher_len)) {
         goto_error(r, TSS2_ESYS_RC_GENERAL_FAILURE, "Encrypt final", cleanup);
     }
     LOGBLOB_TRACE(buffer, buffer_size, "IESYS AES output");
@@ -1028,11 +1028,10 @@ iesys_cryptossl_sym_aes_encrypt(uint8_t * key,
  * @param[in] key_bits Key size in bits.
  * @param[in] tpm_mode Block cipher mode of opertion in TSS2 notation (CFB).
  *            For parameter encryption only CFB can be used.
- * @param[in] blk_len Length Block length of AES.
  * @param[in,out] buffer Data to be decrypted. The decrypted date will be stored
  *                in this buffer.
  * @param[in] buffer_size size of data to be encrypted.
- * @param[in] iv The initialization vector. The size is equal to blk_len.
+ * @param[in] iv The initialization vector.
  * @retval TSS2_RC_SUCCESS on success, or TSS2_ESYS_RC_BAD_VALUE and
  * @retval TSS2_ESYS_RC_BAD_REFERENCE for invalid parameters,
  * @retval TSS2_ESYS_RC_GENERAL_FAILURE for errors of the crypto library.
@@ -1042,7 +1041,6 @@ iesys_cryptossl_sym_aes_decrypt(uint8_t * key,
                                 TPM2_ALG_ID tpm_sym_alg,
                                 TPMI_AES_KEY_BITS key_bits,
                                 TPM2_ALG_ID tpm_mode,
-                                size_t blk_len,
                                 uint8_t * buffer,
                                 size_t buffer_size,
                                 uint8_t * iv)
@@ -1052,9 +1050,6 @@ iesys_cryptossl_sym_aes_decrypt(uint8_t * key,
     EVP_CIPHER_CTX *ctx = NULL;
     int cipher_len = 0;
 
-    /* Parameter blk_len needed for other crypto libraries */
-    (void)blk_len;
-
     if (key == NULL || buffer == NULL) {
         return_error(TSS2_ESYS_RC_BAD_REFERENCE, "Bad reference");
     }
@@ -1084,21 +1079,17 @@ iesys_cryptossl_sym_aes_decrypt(uint8_t * key,
 
     LOGBLOB_TRACE(buffer, buffer_size, "IESYS AES input");
 
-    if (1 != EVP_DecryptInit_ex(ctx, cipher_alg, get_engine(), key, iv)) {
+    if (1 != EVP_DecryptInit(ctx, cipher_alg, key, iv)) {
         goto_error(r, TSS2_ESYS_RC_GENERAL_FAILURE,
                    "Initialize cipher operation", cleanup);
     }
 
-    if (1 != EVP_DecryptInit_ex(ctx, NULL, get_engine(), key, iv)) {
-        goto_error(r, TSS2_ESYS_RC_GENERAL_FAILURE, "Set key and iv", cleanup);
-    }
-
     /* Perform the decryption */
     if (1 != EVP_DecryptUpdate(ctx, buffer, &cipher_len, buffer, buffer_size)) {
         goto_error(r, TSS2_ESYS_RC_GENERAL_FAILURE, "Encrypt update", cleanup);
     }
 
-    if (1 != EVP_DecryptFinal_ex(ctx, buffer, &cipher_len)) {
+    if (1 != EVP_DecryptFinal(ctx, buffer, &cipher_len)) {
         goto_error(r, TSS2_ESYS_RC_GENERAL_FAILURE, "Encrypt final", cleanup);
     }
     LOGBLOB_TRACE(buffer, buffer_size, "IESYS AES output");
@@ -1120,6 +1111,7 @@ iesys_cryptossl_sym_aes_decrypt(uint8_t * key,
  */
 TSS2_RC
 iesys_cryptossl_init() {
+    ENGINE_load_builtin_engines();
     OpenSSL_add_all_algorithms();
     return TSS2_RC_SUCCESS;
 }
