@@ -381,6 +381,8 @@ Esys_TR_SetAuth(ESYS_CONTEXT * esys_context, ESYS_TR esys_handle,
 {
     RSRC_NODE_T *esys_object;
     TSS2_RC r;
+    TPMI_ALG_HASH name_alg = TPM2_ALG_NULL;
+
     _ESYS_ASSERT_NON_NULL(esys_context);
     r = esys_GetResourceObject(esys_context, esys_handle, &esys_object);
     if (r != TPM2_RC_SUCCESS)
@@ -392,7 +394,18 @@ Esys_TR_SetAuth(ESYS_CONTEXT * esys_context, ESYS_TR esys_handle,
         if (authValue->size > sizeof(TPMU_HA)) {
             return_error(TSS2_ESYS_RC_BAD_SIZE, "Bad size for auth value.");
         }
+        /* Determine name alg of resource */
+        if (esys_object->rsrc.rsrcType == IESYSC_KEY_RSRC) {
+            name_alg = esys_object->rsrc.misc.rsrc_key_pub.publicArea.nameAlg;
+        } else if (esys_object->rsrc.rsrcType == IESYSC_NV_RSRC) {
+            name_alg = esys_object->rsrc.misc.rsrc_nv_pub.nvPublic.nameAlg;
+        }
         esys_object->auth = *authValue;
+        /* Adapt auth value to hash for large auth values. */
+        if (name_alg != TPM2_ALG_NULL) {
+            r = iesys_hash_long_auth_values(&esys_object->auth, name_alg);
+            return_if_error(r, "Hashing overlength authValue failed.");
+        }
     }
     return TSS2_RC_SUCCESS;
 }
@@ -408,6 +421,7 @@ Esys_TR_SetAuth(ESYS_CONTEXT * esys_context, ESYS_TR esys_handle,
  * @retval TSS2_ESYS_RC_MEMORY if needed memory can't be allocated.
  * @retval TSS2_ESYS_RC_GENERAL_FAILURE for errors of the crypto library.
  * @retval TSS2_ESYS_RC_BAD_REFERENCE if the esysContext is NULL.
+ * @retval TSS2_ESYS_RC_BAD_TR if the handle is invalid.
  * @retval TSS2_SYS_RC_* for SAPI errors.
  */
 TSS2_RC
@@ -418,6 +432,10 @@ Esys_TR_GetName(ESYS_CONTEXT * esys_context, ESYS_TR esys_handle,
     TSS2_RC r;
     _ESYS_ASSERT_NON_NULL(esys_context);
 
+    if (esys_handle == ESYS_TR_NONE) {
+        return_error(TSS2_ESYS_RC_BAD_TR, "Name for ESYS_TR_NONE can't be determined.");
+    }
+
     r = esys_GetResourceObject(esys_context, esys_handle, &esys_object);
     return_if_error(r, "Object not found");
 
