@@ -254,7 +254,7 @@ ifapi_path_type_p(const char *path, const char *type)
     end_pos = (int)(end - path);
 
     /* Check sub-string and following delimiter. */
-    if (strlen(path) - pos > 3 &&
+    if (strlen(path) - pos >= 3 &&
             strncasecmp(type, &path[pos], strlen(type)) == 0 && end &&
             strncmp(IFAPI_FILE_DELIM, &path[end_pos], 1) == 0)
         return true;
@@ -922,12 +922,51 @@ append_object_to_list(void *object, NODE_OBJECT_T **object_list)
     return TSS2_RC_SUCCESS;
 }
 
+/**  Compute the name of a hierarchy object.
+ *
+ * The TPM handle will be computed from the esys handle and the name
+ * will be computed from the TPM handle.
+ *
+ * @param[in,out] hierarchy The hierarchy object.
+ */
+static void
+set_name_hierarchy_object(IFAPI_OBJECT *object)
+{
+    TPM2_HANDLE handle = 0;
+    size_t offset = 0;
+    switch (object->handle) {
+    case ESYS_TR_RH_NULL:
+        handle = TPM2_RH_NULL;
+        break;
+    case ESYS_TR_RH_OWNER:
+        handle = TPM2_RH_OWNER;
+        break;
+    case ESYS_TR_RH_ENDORSEMENT:
+        handle = TPM2_RH_ENDORSEMENT;
+        break;
+    case ESYS_TR_RH_LOCKOUT:
+        handle = TPM2_RH_LOCKOUT;
+        break;
+    case ESYS_TR_RH_PLATFORM:
+        handle = TPM2_RH_PLATFORM;
+        break;
+    case ESYS_TR_RH_PLATFORM_NV:
+        handle = TPM2_RH_PLATFORM_NV;
+        break;
+    }
+    Tss2_MU_TPM2_HANDLE_Marshal(handle,
+                                &object->misc.hierarchy.name.name[0], sizeof(TPM2_HANDLE),
+                                &offset);
+    object->misc.hierarchy.name.size = offset;
+}
+
 /** Initialize the internal representation of a FAPI hierarchy object.
  *
  * The object will be cleared and the type of the general fapi object will be
  * set to hierarchy.
  *
- * @param[out] hierarchy The caller allocated hierarchy object.
+ * @param[in,out] hierarchy The caller allocated hierarchy object. The name of the
+ *                object will be computed.
  * @param[in] esys_handle The ESAPI handle of the hierarchy which will be added to
  *            to the object.
  */
@@ -940,26 +979,54 @@ ifapi_init_hierarchy_object(
     hierarchy->system = TPM2_YES;
     hierarchy->objectType = IFAPI_HIERARCHY_OBJ;
     hierarchy->handle = esys_handle;
+    hierarchy->misc.hierarchy.esysHandle = esys_handle;
+    set_name_hierarchy_object(hierarchy);
 }
 
-/** Get description of a FAPI object.
+/**  Initialize a hierarchy object read from a file.
  *
- * @param[in] object The object which might have a description.
+ * The esys handles will be set depending on the object path and the
+ * object name will be computed.
  *
- * @retval The character description of the object.
- * @retval NULL if no description is available.
+ * @param[in,out] hierarchy The caller allocated hierarchy object.
+ * @retval TSS2_RC_SUCCESS if the hierarchy could be initialized.
+ * @retval TSS2_FAPI_RC_GENERAL_FAILURE For an invalid hierarchy path.
  */
-char *
-get_description(IFAPI_OBJECT *object)
+TSS2_RC
+ifapi_set_name_hierarchy_object(IFAPI_OBJECT *object)
 {
-    switch (object->objectType) {
-    case IFAPI_KEY_OBJ:
-        return object->misc.key.description;
-    case IFAPI_NV_OBJ:
-        return object->misc.nv.description;
-    default:
-        return NULL;
+    const char *path = object->rel_path;
+    size_t pos = 0, pos2;
+    if (path) {
+        /* Determine esys handle from pathname. */
+        if (strncmp("/", &path[0], 1) == 0)
+            pos += 1;
+        /* Skip profile if it does exist in path */
+        if (strncmp("P_", &path[pos], 2) == 0) {
+            char *  start = strchr(&path[pos], IFAPI_FILE_DELIM_CHAR);
+            if (start) {
+                pos2 = (int)(start - &path[pos]);
+                pos = pos2 + 2;
+            } else {
+                return_error(TSS2_FAPI_RC_GENERAL_FAILURE, "Invalid path.");
+            }
+        }
+        if (strcmp(&path[pos], "HS") == 0) {
+            object->handle = ESYS_TR_RH_OWNER;
+            object->misc.hierarchy.esysHandle = ESYS_TR_RH_OWNER;
+        } else if (strcmp(&path[pos], "HE") == 0) {
+            object->handle = ESYS_TR_RH_ENDORSEMENT;
+            object->misc.hierarchy.esysHandle = ESYS_TR_RH_ENDORSEMENT;
+        } else if (strcmp(&path[pos], "LOCKOUT") == 0) {
+            object->handle = ESYS_TR_RH_LOCKOUT;
+            object->misc.hierarchy.esysHandle = ESYS_TR_RH_LOCKOUT;
+        } else  if (strcmp(&path[pos], "HN") == 0) {
+            object->handle = ESYS_TR_RH_NULL;
+            object->misc.hierarchy.esysHandle = ESYS_TR_RH_NULL;
+        }
     }
+    set_name_hierarchy_object(object);
+    return TSS2_RC_SUCCESS;
 }
 
 /** Create a directory and all sub directories.
@@ -1134,6 +1201,10 @@ ifapi_cleanup_policy(TPMS_POLICY *policy)
         SAFE_FREE(policy->description);
         if (policy->policyAuthorizations) {
             for (size_t i = 0; i < policy->policyAuthorizations->count; i++) {
+                if (strcmp(policy->policyAuthorizations->authorizations[i].type, "pem") == 0) {
+                    SAFE_FREE(policy->policyAuthorizations->authorizations[i].keyPEM);
+                    SAFE_FREE(policy->policyAuthorizations->authorizations[i].pemSignature.buffer);
+                }
                 SAFE_FREE(policy->policyAuthorizations->authorizations[i].type);
             }
         }
@@ -1142,23 +1213,6 @@ ifapi_cleanup_policy(TPMS_POLICY *policy)
     }
 }
 
-/** Free memory of a policy object.
- *
- * The memory allocated during deserialization of the policy will
- * also be freed.
- *
- * @param[in] object The policy object to be cleaned up.
- *
- */
-static void
-cleanup_policy_object(POLICY_OBJECT * object) {
-    if (object != NULL) {
-        SAFE_FREE(object->path);
-        ifapi_cleanup_policy(&object->policy);
-        cleanup_policy_object(object->next);
-    }
-}
-
 static TPML_POLICYELEMENTS *
 copy_policy_elements(const TPML_POLICYELEMENTS *from_policy);
 
@@ -1191,69 +1245,6 @@ error_cleanup:
     return r;
 }
 
-/** Copy policy object.
- *
- * @param[in] src The policy object to be copied.
- * @param[out] dest The destination policy object.
- * @retval TSS2_RC_SUCCESS on success.
- * @retval TSS2_FAPI_RC_MEMORY if not enough memory can be allocated.
- * @retval TSS2_FAPI_RC_BAD_REFERENCE a invalid null pointer is passed.
- */
-static TSS2_RC
-copy_policy_object(POLICY_OBJECT * dest, const POLICY_OBJECT * src) {
-    /* Check for NULL references */
-    if (dest == NULL || src == NULL) {
-        return TSS2_FAPI_RC_MEMORY;
-    }
-
-    TSS2_RC r = TSS2_RC_SUCCESS;
-    dest->policy.description = NULL;
-    dest->policy.policyAuthorizations = NULL;
-    dest->policy.policy = NULL;
-    strdup_check(dest->path, src->path, r, error_cleanup);
-    r = copy_policy(&dest->policy, &src->policy);
-    goto_if_error(r, "Could not copy policy", error_cleanup);
-    if (src->next != NULL) {
-        dest->next = malloc(sizeof(POLICY_OBJECT));
-        goto_if_null(dest->next, "Out of memory", r, error_cleanup);
-        dest->next->next = NULL;
-        r = copy_policy_object(dest->next, src->next);
-        goto_if_error(r, "Could not copy next policy object", error_cleanup);
-    }
-
-    return r;
-error_cleanup:
-    cleanup_policy_object(dest);
-    return r;
-}
-
-/** Copy policy authorization.
- *
- * @param[in] src The policy authorization to be copied.
- * @param[out] dest The destination policy authorization.
- * @retval TSS2_RC_SUCCESS on success.
- * @retval TSS2_FAPI_RC_BAD_REFERENCE a invalid null pointer is passed.
- * @retval TSS2_FAPI_RC_MEMORY if not enough memory can be allocated.
- */
-static TSS2_RC
-copy_policyauthorization(TPMS_POLICYAUTHORIZATION * dest,
-        const TPMS_POLICYAUTHORIZATION * src) {
-    if (dest == NULL || src == NULL) {
-        return TSS2_FAPI_RC_BAD_REFERENCE;
-    }
-    TSS2_RC r = TSS2_RC_SUCCESS;
-    strdup_check(dest->type, src->type, r, error_cleanup);
-
-    dest->key = src->key;
-    dest->policyRef = src->policyRef;
-    dest->signature = src->signature;
-
-    return r;
-error_cleanup:
-    SAFE_FREE(dest->type);
-    return r;
-}
-
 /** Copy policy branches.
  *
  * @param[in] src The policy branches to be copied.
@@ -1336,26 +1327,6 @@ copy_policy_element(const TPMT_POLICYELEMENT *from_policy, TPMT_POLICYELEMENT *t
                      from_policy->element.PolicyAuthorize.keyPath, r, error);
         strdup_check(to_policy->element.PolicyAuthorize.keyPEM,
                      from_policy->element.PolicyAuthorize.keyPEM, r, error);
-        if (from_policy->element.PolicyAuthorize.policy_list) {
-            to_policy->element.PolicyAuthorize.policy_list =
-                malloc(sizeof(POLICY_OBJECT));
-            goto_if_null2(to_policy->element.PolicyAuthorize.policy_list,
-                          "Out of memory", r, TSS2_FAPI_RC_MEMORY, error);
-            to_policy->element.PolicyAuthorize.policy_list->next = NULL;
-            r = copy_policy_object(to_policy->element.PolicyAuthorize.policy_list,
-                                   from_policy->element.PolicyAuthorize.policy_list);
-            goto_if_error(r, "Could not copy policy list", error);
-
-        }
-        if (from_policy->element.PolicyAuthorize.authorization) {
-            to_policy->element.PolicyAuthorize.authorization =
-                malloc(sizeof(TPMS_POLICYAUTHORIZATION));
-            goto_if_null(to_policy->element.PolicyAuthorize.authorization,
-                         "Out of memory", r, error);
-            r = copy_policyauthorization(to_policy->element.PolicyAuthorize.authorization,
-                                         from_policy->element.PolicyAuthorize.authorization);
-            goto_if_error(r, "Could not copy policy authorization", error);
-        }
         break;
     case POLICYAUTHORIZENV:
         strdup_check(to_policy->element.PolicyAuthorizeNv.nvPath,
@@ -1390,6 +1361,10 @@ copy_policy_element(const TPMT_POLICYELEMENT *from_policy, TPMT_POLICYELEMENT *t
                      from_policy->element.PolicyDuplicationSelect.newParentPath,
                      r, error);
         break;
+    case POLICYACTION:
+        strdup_check(to_policy->element.PolicyAction.action,
+                     from_policy->element.PolicyAction.action, r, error);
+        break;
     case POLICYNAMEHASH:
         for (size_t i = 0; i < from_policy->element.PolicyNameHash.count; i++) {
             strdup_check(to_policy->element.PolicyNameHash.namePaths[i],
@@ -1397,12 +1372,6 @@ copy_policy_element(const TPMT_POLICYELEMENT *from_policy, TPMT_POLICYELEMENT *t
                     r, error);
         }
         break;
-    case POLICYOR:
-        to_policy->element.PolicyOr.branches =
-            copy_policy_branches(from_policy->element.PolicyOr.branches);
-        goto_if_null2(to_policy->element.PolicyOr.branches, "Out of memory",
-                      r, TSS2_FAPI_RC_MEMORY, error);
-        break;
     }
     return TSS2_RC_SUCCESS;
 
@@ -1634,6 +1603,9 @@ ifapi_object_cmp_name(IFAPI_OBJECT *object, void *name, bool *equal)
     TPM2B_NAME nv_name;
 
     switch (object->objectType) {
+    case IFAPI_HIERARCHY_OBJ:
+        obj_name = &object->misc.hierarchy.name;
+        break;
     case IFAPI_KEY_OBJ:
         obj_name = &object->misc.key.name;
         break;
@@ -2591,3 +2563,40 @@ out_global_cleanup:
 out_memory:
     return ret;
 }
+
+/** Check valid keys of a json object.
+ *
+ * @param[in]  jso The json object.
+ * @param[out] field_tab the array of strings with allowed fields.
+ * @param[out] size_of_tab The number of allowed fields.
+ *
+ * If a unexpected field occurs a warning will be displayed.
+ */
+void
+ifapi_check_json_object_fields(
+    json_object *jso,
+    char** field_tab,
+    size_t size_of_tab)
+{
+    enum json_type type;
+    bool found;
+    size_t i;
+
+    type = json_object_get_type(jso);
+    if (type == json_type_object) {
+        /* Object with keys. */
+        json_object_object_foreach(jso, key, val) {
+            UNUSED(val);
+            found = false;
+            for (i = 0; i < size_of_tab; i++) {
+                if (strcmp(key, field_tab[i]) == 0) {
+                    found = true;
+                    break;
+                }
+            }
+            if (!found) {
+                LOG_WARNING("Invalid field: %s", key);
+            }
+        }
+    }
+}
