@@ -17,6 +17,7 @@
 #include <errno.h>
 #include <sys/types.h>
 #include <dirent.h>
+#include <limits.h>
 /* Need for some libc-versions */
 #ifndef __FreeBSD__
 #include <malloc.h>
@@ -43,6 +44,9 @@ ifapi_io_read_async(
     struct IFAPI_IO *io,
     const char *filename)
 {
+    struct stat statbuf;
+    struct flock flock  = { 0 };
+
     if (io->char_rbuffer) {
         LOG_ERROR("rbuffer still in use; maybe use of old API.");
         return TSS2_FAPI_RC_IO_ERROR;
@@ -50,21 +54,53 @@ ifapi_io_read_async(
 
     io->stream = fopen(filename, "rt");
     if (io->stream == NULL) {
-        LOG_ERROR("File \"%s\" not found.", filename);
+        LOG_ERROR("Open file \"%s\": %s", filename, strerror(errno));
         return TSS2_FAPI_RC_IO_ERROR;
     }
-    /* Locking the file. Lock will be release upon close */
-    if (lockf(fileno(io->stream), F_TLOCK, 0) == -1 && errno == EAGAIN) {
-        LOG_ERROR("File %s currently locked.", filename);
+
+    if (fstat(fileno(io->stream), &statbuf) == -1) {
+        fclose(io->stream);
+        LOG_ERROR("Execute fstat for \"%s\".", filename);
+        return TSS2_FAPI_RC_IO_ERROR;
+    }
+
+    /* Check whether file is a directory. */
+    if (S_ISDIR(statbuf.st_mode)) {
+        fclose(io->stream);
+        LOG_ERROR("\"%s\" is a directory.", filename);
+        return TSS2_FAPI_RC_IO_ERROR;
+    }
+
+    /* Locking the file. Lock will be released upon close */
+    flock.l_type = F_RDLCK;
+    flock.l_whence = SEEK_SET;
+
+    if (fcntl(fileno(io->stream), F_SETLK, &flock) == -1) {
+        fclose(io->stream);
+        LOG_ERROR("File \"%s\" could not be locked: %s",
+                  filename, strerror(errno));
         fclose(io->stream);
         return TSS2_FAPI_RC_IO_ERROR;
     }
 
-    fseek(io->stream, 0L, SEEK_END);
+    if (fseek(io->stream, 0L, SEEK_END) == -1) {
+        LOG_ERROR("fseek failed for \"%s\".", filename);
+        fclose(io->stream);
+        return TSS2_FAPI_RC_IO_ERROR;
+    };
     long length = ftell(io->stream);
+    if (length == -1  || length == LONG_MAX) {
+        LOG_ERROR("ftell failed for \"%s\".", filename);
+        fclose(io->stream);
+        return TSS2_FAPI_RC_IO_ERROR;
+    };
     fclose(io->stream);
 
     io->stream = fopen(filename, "rt");
+    if (io->stream == NULL) {
+        LOG_ERROR("Open file \"%s\": %s", filename, strerror(errno));
+        return TSS2_FAPI_RC_IO_ERROR;
+    }
     io->char_rbuffer = malloc (length + 1);
     if (io->char_rbuffer == NULL) {
         fclose(io->stream);
@@ -73,9 +109,14 @@ ifapi_io_read_async(
         return TSS2_FAPI_RC_MEMORY;
     }
 
-    int rc, flags = fcntl(fileno(io->stream), F_GETFL, 0);
-    rc = fcntl(fileno(io->stream), F_SETFL, flags | O_NONBLOCK);
-    if (rc < 0) {
+    int flags = fcntl(fileno(io->stream), F_GETFL, 0);
+    if (flags == -1) {
+        SAFE_FREE(io->char_rbuffer);
+        LOG_ERROR("fcntl failed with %d", errno);
+        return TSS2_FAPI_RC_IO_ERROR;
+    }
+    if (fcntl(fileno(io->stream), F_SETFL, flags | O_NONBLOCK) == -1) {
+        SAFE_FREE(io->char_rbuffer);
         LOG_ERROR("fcntl failed with %d", errno);
         return TSS2_FAPI_RC_IO_ERROR;
     }
@@ -162,6 +203,9 @@ ifapi_io_write_async(
     const uint8_t *buffer,
     size_t length)
 {
+    TSS2_RC r;
+    struct flock flock = { 0 };
+
     if (io->char_rbuffer) {
         LOG_ERROR("rbuffer still in use; maybe use of old API.");
         return TSS2_FAPI_RC_IO_ERROR;
@@ -178,25 +222,39 @@ ifapi_io_write_async(
 
     io->stream = fopen(filename, "wt");
     if (io->stream == NULL) {
-        SAFE_FREE(io->char_rbuffer);
-        LOG_ERROR("Could not open file \"%s\" for writing.", filename);
-        return TSS2_FAPI_RC_IO_ERROR;
+        goto_error(r, TSS2_FAPI_RC_IO_ERROR,
+                   "Open file \"%s\" for writing: %s", error, filename,
+                   strerror(errno));
     }
-    /* Locking the file. Lock will be release upon close */
-    if (lockf(fileno(io->stream), F_TLOCK, 0) == -1 && errno == EAGAIN) {
-        LOG_ERROR("File %s currently locked.", filename);
+    /* Locking the file. Lock will be released upon close */
+    flock.l_type = F_WRLCK;
+    flock.l_whence = SEEK_SET;
+
+    if (fcntl(fileno(io->stream), F_SETLK, &flock) == -1) {
         fclose(io->stream);
-        return TSS2_FAPI_RC_IO_ERROR;
+        goto_error(r, TSS2_FAPI_RC_IO_ERROR,
+                   "File \"%s\" could not be locked: %s", error, filename,
+                   strerror(errno));
     }
 
     /* Use non blocking IO, so asynchronous write will be needed */
     int rc, flags = fcntl(fileno(io->stream), F_GETFL, 0);
+    if (flags < 0) {
+        fclose(io->stream);
+        goto_error(r, TSS2_FAPI_RC_IO_ERROR,
+                   "fcntl failed with %d", error, errno);
+    }
     rc = fcntl(fileno(io->stream), F_SETFL, flags | O_NONBLOCK);
     if (rc < 0) {
-        LOG_ERROR("fcntl failed with %d", errno);
-        return TSS2_FAPI_RC_IO_ERROR;
+        fclose(io->stream);
+        goto_error(r, TSS2_FAPI_RC_IO_ERROR,
+                   "fcntl failed with %d", error, errno);
     }
     return TSS2_RC_SUCCESS;
+
+ error:
+    SAFE_FREE(io->char_rbuffer);
+    return r;
 }
 
 /** Finish writing a buffer into a file in an asynchronous way.
